@@ -46,9 +46,11 @@ settingsDialogImpl::settingsDialogImpl(QWidget *parent, ConfigFile *c, selectAva
 
 	comboBox_switchLanguage->addItem(tr("Bulgarian"),"bg");
 	comboBox_switchLanguage->addItem(tr("Chinese"),"zhcn");
+	comboBox_switchLanguage->addItem(tr("Danish"),"dk");
 	comboBox_switchLanguage->addItem(tr("Dutch"),"nl");
 	comboBox_switchLanguage->addItem(tr("English"),"en");
 	comboBox_switchLanguage->addItem(tr("French"),"fr");
+	comboBox_switchLanguage->addItem(tr("Finnish"),"fi");
 	comboBox_switchLanguage->addItem(tr("German"),"de");
 	comboBox_switchLanguage->addItem(tr("Greek"),"gr");
 	comboBox_switchLanguage->addItem(tr("Hungarian"),"hu");
@@ -92,6 +94,10 @@ settingsDialogImpl::settingsDialogImpl(QWidget *parent, ConfigFile *c, selectAva
 	connect( radioButton_manualBlindsOrder, SIGNAL( toggled(bool) ), this, SLOT( setFirstSmallBlindMargin() ));	//temporarely unused until ai is enabled in network
 
 	connect( comboBox_switchLanguage, SIGNAL( currentIndexChanged (int) ), this, SLOT ( setLanguageChanged(int) ));
+
+	connect(groupBox_automaticServerConfig, SIGNAL(toggled(bool)), this, SLOT(toggleGroupBoxAutomaticServerConfig(bool)));
+	connect(groupBox_manualServerConfig, SIGNAL(toggled(bool)), this, SLOT(toggleGroupBoxManualServerConfig(bool)));
+
 // 	label_36->hide();
 // 	spinBox_netGameSpeed->hide();
 // 
@@ -162,7 +168,10 @@ void settingsDialogImpl::exec() {
 	checkBox_useIpv6->setChecked(myConfig->readConfigInt("ServerUseIpv6"));
 	checkBox_useSctp->setChecked(myConfig->readConfigInt("ServerUseSctp"));
 
-	//Internet Game Settings
+	//Internet Game Settings 
+	if(myConfig->readConfigInt("InternetServerConfigMode")) { groupBox_manualServerConfig->setChecked(TRUE); }
+	else { groupBox_automaticServerConfig->setChecked(TRUE); }
+	lineEdit_InternetServerListAddress->setText(QString::fromUtf8(myConfig->readConfigString("InternetServerListAddress").c_str()));
 	lineEdit_InternetServerAddress->setText(QString::fromUtf8(myConfig->readConfigString("InternetServerAddress").c_str()));
 	spinBox_InternetServerPort->setValue(myConfig->readConfigInt("InternetServerPort"));
 	lineEdit_InternetServerPassword->setText(QString::fromUtf8(myConfig->readConfigString("InternetServerPassword").c_str()));
@@ -180,7 +189,6 @@ void settingsDialogImpl::exec() {
 	checkBox_IRCServerUseIpv6->setChecked(myConfig->readConfigInt("IRCServerUseIpv6"));
 
 	//Interface
-
 	comboBox_switchLanguage->setCurrentIndex(comboBox_switchLanguage->findData(QString::fromUtf8(myConfig->readConfigString("Language").c_str()).section('_', 0, 0)));
 	checkBox_showLeftToolbox->setChecked(myConfig->readConfigInt("ShowLeftToolBox"));
 	checkBox_showRightToolbox->setChecked(myConfig->readConfigInt("ShowRightToolBox"));
@@ -192,6 +200,7 @@ void settingsDialogImpl::exec() {
 	checkBox_antiPeekMode->setChecked(myConfig->readConfigInt("AntiPeekMode"));
 	checkBox_alternateFKeysUserActionMode->setChecked(myConfig->readConfigInt("AlternateFKeysUserActionMode"));
 	checkBox_enableBetInputFocusSwitch->setChecked(myConfig->readConfigInt("EnableBetInputFocusSwitch"));
+	checkBox_cardsChanceMonitor->setChecked(myConfig->readConfigInt("ShowCardsChanceMonitor"));
 
 	radioButton_flipsideTux->setChecked(myConfig->readConfigInt("FlipsideTux"));
 	radioButton_flipsideOwn->setChecked(myConfig->readConfigInt("FlipsideOwn"));
@@ -303,9 +312,34 @@ void settingsDialogImpl::isAccepted() {
 	myConfig->writeConfigInt("ServerUseSctp", checkBox_useSctp->isChecked());
 
 	//Internet Game Settings
-	myConfig->writeConfigString("InternetServerAddress", lineEdit_InternetServerAddress->text().toUtf8().constData());
-	myConfig->writeConfigInt("InternetServerPort", spinBox_InternetServerPort->value());
-	myConfig->writeConfigString("InternetServerPassword", lineEdit_InternetServerPassword->text().toUtf8().constData());
+	if(groupBox_automaticServerConfig->isChecked()) { 
+
+		myConfig->writeConfigInt("InternetServerConfigMode", 0); 
+
+// 		QUrl serverList;
+// 		serverList.setUrl(lineEdit_InternetServerListAddress->text(),QUrl::StrictMode);
+// 		QRegExp rx("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?");
+// 		if(lineEdit_InternetServerListAddress->text().contains(rx)) {		
+		if(lineEdit_InternetServerListAddress->text().contains("/") && lineEdit_InternetServerListAddress->text().contains(".") ) {
+
+			myConfig->writeConfigString("InternetServerListAddress", lineEdit_InternetServerListAddress->text().toUtf8().constData());
+		}
+		else {
+			QMessageBox::warning(this, tr("Settings Error"),
+				tr("The entered server list address is not a valid URL.\n"
+				"Please enter a valid server list address!"),
+				QMessageBox::Ok);
+				settingsCorrect = FALSE; 
+		}
+	}
+	if(groupBox_manualServerConfig->isChecked()) { 
+		
+		myConfig->writeConfigInt("InternetServerConfigMode", 1); 
+		myConfig->writeConfigString("InternetServerAddress", lineEdit_InternetServerAddress->text().toUtf8().constData());
+		myConfig->writeConfigInt("InternetServerPort", spinBox_InternetServerPort->value());
+		myConfig->writeConfigString("InternetServerPassword", lineEdit_InternetServerPassword->text().toUtf8().constData());
+	}
+
 	myConfig->writeConfigInt("InternetServerUseIpv6", checkBox_InternetServerUseIpv6->isChecked());
 	myConfig->writeConfigInt("InternetServerUseSctp", checkBox_InternetServerUseSctp->isChecked());
 	myConfig->writeConfigInt("UseInternetGamePassword", checkBox_UseInternetGamePassword->isChecked());
@@ -326,6 +360,7 @@ void settingsDialogImpl::isAccepted() {
 	myConfig->writeConfigInt("ShowFadeOutCardsAnimation", checkBox_showFadeOutCardsAnimation->isChecked());
 	myConfig->writeConfigInt("ShowFlipCardsAnimation", checkBox_showFlipCardsAnimation->isChecked());
 	myConfig->writeConfigInt("ShowBlindButtons", checkBox_showBlindButtons->isChecked());
+	myConfig->writeConfigInt("ShowCardsChanceMonitor", checkBox_cardsChanceMonitor->isChecked());
 	myConfig->writeConfigInt("AntiPeekMode", checkBox_antiPeekMode->isChecked());
 	myConfig->writeConfigInt("AlternateFKeysUserActionMode", checkBox_alternateFKeysUserActionMode->isChecked());
 	myConfig->writeConfigInt("EnableBetInputFocusSwitch", checkBox_enableBetInputFocusSwitch->isChecked());
@@ -391,7 +426,7 @@ void settingsDialogImpl::isAccepted() {
 	}
 
 	//Wenn alles richtig eingegeben wurde --> Dialog schließen
-	if(settingsCorrect) { this->hide(); }
+	if(settingsCorrect) { this->accept(); }
 }
 
 void settingsDialogImpl::setFlipsidePicFileName()
@@ -603,3 +638,14 @@ void settingsDialogImpl::setLanguageChanged(int index) {
 	changedLanguageIndex = index;
 	
 }
+
+void settingsDialogImpl::toggleGroupBoxAutomaticServerConfig(bool toggleState) { 
+	if(toggleState) groupBox_manualServerConfig->setChecked(FALSE); 
+	else groupBox_manualServerConfig->setChecked(TRUE); 
+}
+
+void settingsDialogImpl::toggleGroupBoxManualServerConfig(bool toggleState) { 
+	if(toggleState) groupBox_automaticServerConfig->setChecked(FALSE); 
+	else groupBox_automaticServerConfig->setChecked(TRUE); 
+}
+
