@@ -702,7 +702,7 @@ AbstractClientStateReceiving::Process(ClientThread &client)
 			tmpPacket->ToNetPacketPlayerLeft()->GetData(playerLeftData);
 
 			// Signal to GUI and remove from data list.
-			client.RemovePlayerData(playerLeftData.playerId);
+			client.RemovePlayerData(playerLeftData.playerId, playerLeftData.removeReason);
 		}
 		else if (tmpPacket->ToNetPacketGameAdminChanged())
 		{
@@ -769,6 +769,26 @@ AbstractClientStateReceiving::Process(ClientThread &client)
 			tmpPacket->ToNetPacketGameListAdminChanged()->GetData(adminChangedData);
 			client.UpdateGameInfoAdmin(adminChangedData.gameId, adminChangedData.newAdminplayerId);
 		}
+		else if (tmpPacket->ToNetPacketStartKickPlayerPetition())
+		{
+			NetPacketStartKickPlayerPetition::Data startPetitionData;
+			tmpPacket->ToNetPacketStartKickPlayerPetition()->GetData(startPetitionData);
+			client.StartPetition(startPetitionData.petitionId, startPetitionData.proposingPlayerId,
+				startPetitionData.kickPlayerId, startPetitionData.kickTimeoutSec, startPetitionData.numVotesNeededToKick);
+		}
+		else if (tmpPacket->ToNetPacketKickPlayerPetitionUpdate())
+		{
+			NetPacketKickPlayerPetitionUpdate::Data updatePetitionData;
+			tmpPacket->ToNetPacketKickPlayerPetitionUpdate()->GetData(updatePetitionData);
+			client.UpdatePetition(updatePetitionData.petitionId, updatePetitionData.numVotesAgainstKicking,
+				updatePetitionData.numVotesInFavourOfKicking, updatePetitionData.numVotesNeededToKick);
+		}
+		else if (tmpPacket->ToNetPacketEndKickPlayerPetition())
+		{
+			NetPacketEndKickPlayerPetition::Data endPetitionData;
+			tmpPacket->ToNetPacketEndKickPlayerPetition()->GetData(endPetitionData);
+			client.EndPetition(endPetitionData.petitionId);
+		}
 		else if (tmpPacket->ToNetPacketAvatarHeader())
 		{
 			NetPacketAvatarHeader::Data headerData;
