@@ -38,8 +38,6 @@ using namespace std;
 
 //#define SERVER_TEST
 
-#define SERVER_START_GAME_TIMEOUT_SEC			10
-
 #ifdef SERVER_TEST
 	#define SERVER_DELAY_NEXT_HAND_SEC				0
 	#define SERVER_DELAY_NEXT_GAME_SEC				0
@@ -62,8 +60,10 @@ using namespace std;
 	#define SERVER_COMPUTER_ACTION_DELAY_SEC		2
 #endif
 
+#define SERVER_START_GAME_TIMEOUT_SEC				10
 #define SERVER_GAME_ADMIN_WARNING_REMAINING_SEC		60
 #define SERVER_GAME_ADMIN_TIMEOUT_SEC				300		// 5 min, MUST be > SERVER_GAME_ADMIN_WARNING_REMAINING_SEC
+#define SERVER_VOTE_KICK_TIMEOUT_SEC				30
 
 // Helper functions
 // TODO: these are hacks.
@@ -152,7 +152,7 @@ AbstractServerGameStateReceiving::Process(ServerGameThread &server)
 	int retVal = MSG_SOCK_INTERNAL_PENDING;
 	SessionWrapper session = server.GetSessionManager().Select(RECV_TIMEOUT_MSEC);
 
-	if (session.sessionData.get())
+	if (session.sessionData)
 	{
 		boost::shared_ptr<NetPacket> packet;
 		try
@@ -166,7 +166,7 @@ AbstractServerGameStateReceiving::Process(ServerGameThread &server)
 		}
 
 		// Process packet if one was received.
-		if (packet.get())
+		if (packet)
 		{
 			if (packet->IsClientActivity())
 				session.sessionData->ResetActivityTimer();
@@ -186,8 +186,9 @@ AbstractServerGameStateReceiving::Process(ServerGameThread &server)
 			}
 			else if (packet->ToNetPacketKickPlayer())
 			{
-				// Only admins are allowed to kick.
-				if (session.playerData->GetRights() == PLAYER_RIGHTS_ADMIN)
+				// Only admins are allowed to kick, and only in the lobby.
+				// After leaving the lobby, a vote needs to be initiated to kick.
+				if (session.playerData->GetRights() == PLAYER_RIGHTS_ADMIN && !server.IsRunning())
 				{
 					NetPacketKickPlayer::Data kickPlayerData;
 					packet->ToNetPacketKickPlayer()->GetData(kickPlayerData);
@@ -195,6 +196,26 @@ AbstractServerGameStateReceiving::Process(ServerGameThread &server)
 					server.InternalKickPlayer(kickPlayerData.playerId);
 				}
 			}
+			else if (packet->ToNetPacketAskKickPlayer())
+			{
+				if (session.playerData)
+				{
+					NetPacketAskKickPlayer::Data askKickData;
+					packet->ToNetPacketAskKickPlayer()->GetData(askKickData);
+
+					server.InternalAskVoteKick(session, askKickData.playerId, SERVER_VOTE_KICK_TIMEOUT_SEC);
+				}
+			}
+			else if (packet->ToNetPacketVoteKickPlayer())
+			{
+				if (session.playerData)
+				{
+					NetPacketVoteKickPlayer::Data voteData;
+					packet->ToNetPacketVoteKickPlayer()->GetData(voteData);
+
+					server.InternalVoteKick(session, voteData.petitionId, voteData.vote);
+				}
+			}
 			// Chat text is always allowed.
 			else if (packet->ToNetPacketSendChatText())
 			{
@@ -213,6 +234,18 @@ AbstractServerGameStateReceiving::Process(ServerGameThread &server)
 					server.SendToAllPlayers(outChat, SessionData::Game);
 				}
 			}
+			else if (packet->ToNetPacketUnsubscribeGameList())
+			{
+				// We can do this directly in this thread.
+				session.sessionData->ResetWantsLobbyMsg();
+			}
+			else if (packet->ToNetPacketResubscribeGameList())
+			{
+				// This needs to be performed in the lobby thread,
+				// because a new game list needs to be sent.
+				if (!session.sessionData->WantsLobbyMsg())
+					server.GetLobbyThread().ResubscribeLobbyMsg(session);
+			}
 			else
 			{
 				// Packet processing in subclass.
@@ -458,6 +491,7 @@ ServerGameStateWaitAck::Process(ServerGameThread &server)
 	if (server.GetStateTimer().elapsed().total_seconds() >= SERVER_START_GAME_TIMEOUT_SEC)
 	{
 		// On timeout: start anyway.
+		server.GetSessionManager().ResetAllReadyFlags();
 		server.SetState(SERVER_START_GAME_STATE::Instance());
 		retVal = MSG_SOCK_INIT_DONE;
 	}
@@ -509,33 +543,49 @@ ServerGameStateStartGame::~ServerGameStateStartGame()
 int
 ServerGameStateStartGame::Process(ServerGameThread &server)
 {
-	server.InternalStartGame();
+	int retVal = MSG_SOCK_INTERNAL_PENDING;
 
-	boost::shared_ptr<NetPacket> answer(new NetPacketGameStart);
-
-	NetPacketGameStart::Data gameStartData;
-	gameStartData.startData = server.GetStartData();
-
-	// Send player order to clients.
-	// Assume player list is sorted by number.
 	PlayerDataList tmpPlayerList = server.GetFullPlayerDataList();
-	PlayerDataList::iterator player_i = tmpPlayerList.begin();
-	PlayerDataList::iterator player_end = tmpPlayerList.end();
-	while (player_i != player_end)
+	if (tmpPlayerList.size() <= 1)
 	{
-		NetPacketGameStart::PlayerSlot tmpPlayerSlot;
-		tmpPlayerSlot.playerId = (*player_i)->GetUniqueId();
-		gameStartData.playerSlots.push_back(tmpPlayerSlot);
+		if (!tmpPlayerList.empty())
+		{
+			boost::shared_ptr<PlayerData> tmpPlayer(tmpPlayerList.front());
+			SessionWrapper tmpSession = server.GetSessionManager().GetSessionByUniquePlayerId(tmpPlayer->GetUniqueId());
+			if (tmpSession.sessionData.get())
+				server.MoveSessionToLobby(tmpSession, NTF_NET_REMOVED_START_FAILED);
+		}
+	}
+	else
+	{
+		server.InternalStartGame();
 
-		++player_i;
+		boost::shared_ptr<NetPacket> answer(new NetPacketGameStart);
+
+		NetPacketGameStart::Data gameStartData;
+		gameStartData.startData = server.GetStartData();
+
+		// Send player order to clients.
+		// Assume player list is sorted by number.
+		PlayerDataList::iterator player_i = tmpPlayerList.begin();
+		PlayerDataList::iterator player_end = tmpPlayerList.end();
+		while (player_i != player_end)
+		{
+			NetPacketGameStart::PlayerSlot tmpPlayerSlot;
+			tmpPlayerSlot.playerId = (*player_i)->GetUniqueId();
+			gameStartData.playerSlots.push_back(tmpPlayerSlot);
+	
+			++player_i;
+		}
+
+		static_cast<NetPacketGameStart *>(answer.get())->SetData(gameStartData);
+
+		server.SendToAllPlayers(answer, SessionData::Game);
+		server.SetState(ServerGameStateStartHand::Instance());
+		retVal = MSG_NET_GAME_SERVER_START; 
 	}
 
-	static_cast<NetPacketGameStart *>(answer.get())->SetData(gameStartData);
-
-	server.SendToAllPlayers(answer, SessionData::Game);
-	server.SetState(ServerGameStateStartHand::Instance());
-
-	return MSG_NET_GAME_SERVER_START;
+	return retVal;
 }
 
 //-----------------------------------------------------------------------------
