@@ -31,21 +31,28 @@
 
 #include <boost/bind.hpp>
 
+
+#define SERVER_CHECK_VOTE_KICK_INTERVAL_MSEC	500
+#define SERVER_KICK_TIMEOUT_ADD_DELAY_SEC		2
+
 using namespace std;
 
 
 ServerGameThread::ServerGameThread(ServerLobbyThread &lobbyThread, u_int32_t id, const string &name, const string &pwd, const GameData &gameData, unsigned adminPlayerId, GuiInterface &gui, ConfigFile *playerConfig)
 : m_adminPlayerId(adminPlayerId), m_lobbyThread(lobbyThread), m_gui(gui),
   m_gameData(gameData), m_id(id), m_name(name), m_password(pwd), m_playerConfig(playerConfig),
-  m_curState(NULL), m_gameNum(1),
+  m_curState(NULL), m_gameNum(1), m_curPetitionId(1),
   m_stateTimer(boost::posix_time::time_duration(0, 0, 0), boost::timers::portable::microsec_timer::manual_start),
   m_stateTimerFlag(0)
 {
+	LOG_VERBOSE("Game object " << GetId() << " created.");
+
 	m_receiver.reset(new ReceiverHelper);
 }
 
 ServerGameThread::~ServerGameThread()
 {
+	LOG_VERBOSE("Game object " << GetId() << " destructed.");
 }
 
 u_int32_t
@@ -98,6 +105,7 @@ ServerGameThread::RemoveAllSessions()
 	{
 		SessionWrapper tmpSession = m_sessionQueue.front();
 		m_sessionQueue.pop_front();
+		LOG_VERBOSE("Game closing, forcing removal of session #" << tmpSession.sessionData->GetId() << ".");
 		lobbyThread.RemoveSessionFromGame(tmpSession);
 	}
 	GetSessionManager().ForEach(boost::bind(&ServerLobbyThread::RemoveSessionFromGame, boost::ref(lobbyThread), _1));
@@ -106,6 +114,8 @@ ServerGameThread::RemoveAllSessions()
 void
 ServerGameThread::Main()
 {
+	LOG_VERBOSE("Game thread " << GetId() << " started.");
+
 	SetState(SERVER_INITIAL_STATE::Instance());
 
 	try
@@ -129,6 +139,7 @@ ServerGameThread::Main()
 			// Process current state.
 			GetState().Process(*this);
 			RemovePlayerLoop();
+			VoteKickLoop();
 		} while (!ShouldTerminate() && GetSessionManager().HasSessions());
 	} catch (const PokerTHException &e)
 	{
@@ -138,6 +149,8 @@ ServerGameThread::Main()
 
 	ResetComputerPlayerList();
 	GetLobbyThread().RemoveGame(GetId());
+
+	LOG_VERBOSE("Game thread " << GetId() << " terminating.");
 }
 
 void
@@ -159,6 +172,87 @@ ServerGameThread::RemovePlayerLoop()
 	m_removePlayerList.clear();
 }
 
+void
+ServerGameThread::VoteKickLoop()
+{
+	// Do not call the vote kick action all the time.
+	// Check the timer.
+	if (m_voteKickActionTimer.elapsed().total_milliseconds() >= SERVER_CHECK_VOTE_KICK_INTERVAL_MSEC)
+		VoteKickAction();
+}
+
+void
+ServerGameThread::VoteKickAction()
+{
+	// Check whether someone should be kicked, or whether a vote kick should be aborted.
+	// Only one vote kick can be active at a time.
+	boost::mutex::scoped_lock lock(m_voteKickDataMutex);
+	if (m_voteKickData)
+	{
+		// Prepare some values.
+		const PlayerIdList playerIds(GetPlayerIdList());
+		int votesRequiredToKick = m_voteKickData->numVotesToKick - m_voteKickData->numVotesInFavourOfKicking;
+		int playersAllowedToVote = 0;
+		// We need to count the number of players which are still allowed to vote.
+		PlayerIdList::const_iterator player_i = playerIds.begin();
+		PlayerIdList::const_iterator player_end = playerIds.end();
+		while (player_i != player_end)
+		{
+			if (find(m_voteKickData->votedPlayerIds.begin(), m_voteKickData->votedPlayerIds.end(), *player_i) == m_voteKickData->votedPlayerIds.end())
+				playersAllowedToVote++;
+			++player_i;
+		}
+		bool abortPetition = false;
+		bool doKick = false;
+		EndPetitionReason reason;
+
+		// 1. Enough votes to kick the player.
+		if (m_voteKickData->numVotesInFavourOfKicking >= m_voteKickData->numVotesToKick)
+		{
+			reason = PETITION_END_ENOUGH_VOTES;
+			abortPetition = true;
+			doKick = true;
+		}
+		// 2. Several players left the game, so a kick is no longer possible.
+		else if (votesRequiredToKick > playersAllowedToVote)
+		{
+			reason = PETITION_END_NOT_ENOUGH_PLAYERS;
+			abortPetition = true;
+		}
+		// 3. The kick has become invalid because the player to be kicked left.
+		else if (!IsValidPlayer(m_voteKickData->kickPlayerId))
+		{
+			reason = PETITION_END_PLAYER_LEFT;
+			abortPetition = true;
+		}
+		// 4. A kick request timed out (because not everyone voted).
+		else if (m_voteKickData->voteTimer.elapsed().total_seconds() >= m_voteKickData->timeLimitSec)
+		{
+			reason = PETITION_END_TIMEOUT;
+			abortPetition = true;
+		}
+		if (abortPetition)
+		{
+			boost::shared_ptr<NetPacket> endPetition(new NetPacketEndKickPlayerPetition);
+			NetPacketEndKickPlayerPetition::Data endPetitionData;
+			endPetitionData.petitionId = m_voteKickData->petitionId;
+			endPetitionData.numVotesAgainstKicking = m_voteKickData->numVotesAgainstKicking;
+			endPetitionData.numVotesInFavourOfKicking = m_voteKickData->numVotesInFavourOfKicking;
+			endPetitionData.playerKicked = doKick;
+			endPetitionData.endReason = reason;
+
+			static_cast<NetPacketEndKickPlayerPetition *>(endPetition.get())->SetData(endPetitionData);
+			SendToAllPlayers(endPetition, SessionData::Game);
+
+			// Perform kick.
+			if (doKick)
+				InternalKickPlayer(m_voteKickData->kickPlayerId);
+			// This petition has ended.
+			m_voteKickData.reset();
+		}
+	}
+}
+
 void
 ServerGameThread::InternalStartGame()
 {
@@ -216,6 +310,124 @@ ServerGameThread::InternalKickPlayer(unsigned playerId)
 	// Only kick if the player was found.
 	if (tmpSession.sessionData.get())
 		MoveSessionToLobby(tmpSession, NTF_NET_REMOVED_KICKED);
+	else
+	{
+		boost::shared_ptr<PlayerData> tmpData(RemoveComputerPlayer(playerId));
+		if (tmpData)
+			RemovePlayerData(tmpData, NTF_NET_REMOVED_KICKED);
+	}
+}
+
+void
+ServerGameThread::InternalAskVoteKick(SessionWrapper byWhom, unsigned playerIdWho, unsigned timeoutSec)
+{
+	if (IsRunning() && byWhom.playerData)
+	{
+		// Retrieve only the number of human players.
+		size_t numPlayers = GetSessionManager().GetPlayerIdList().size();
+		if (numPlayers > 2)
+		{
+			// Check whether the player to be kicked exists.
+			if (IsValidPlayer(playerIdWho))
+			{
+				// Lock the vote kick data.
+				boost::mutex::scoped_lock lock(m_voteKickDataMutex);
+				if (!m_voteKickData)
+				{
+					// Initiate a vote kick.
+					unsigned playerIdByWhom = byWhom.playerData->GetUniqueId();
+					m_voteKickData.reset(new VoteKickData);
+					m_voteKickData->petitionId = m_curPetitionId++;
+					m_voteKickData->kickPlayerId = playerIdWho;
+					m_voteKickData->numVotesToKick = static_cast<int>(ceil(numPlayers / 3. * 2.));
+					m_voteKickData->timeLimitSec = timeoutSec + SERVER_KICK_TIMEOUT_ADD_DELAY_SEC;
+					// Consider first vote.
+					m_voteKickData->numVotesInFavourOfKicking = 1;
+					m_voteKickData->votedPlayerIds.push_back(playerIdByWhom);
+
+					boost::shared_ptr<NetPacket> startPetition(new NetPacketStartKickPlayerPetition);
+					NetPacketStartKickPlayerPetition::Data startPetitionData;
+					startPetitionData.petitionId = m_voteKickData->petitionId;
+					startPetitionData.proposingPlayerId = playerIdByWhom;
+					startPetitionData.kickPlayerId = m_voteKickData->kickPlayerId;
+					startPetitionData.kickTimeoutSec = timeoutSec;
+					startPetitionData.numVotesNeededToKick = m_voteKickData->numVotesToKick;
+
+					static_cast<NetPacketStartKickPlayerPetition *>(startPetition.get())->SetData(startPetitionData);
+					SendToAllPlayers(startPetition, SessionData::Game);
+				}
+				else
+					InternalDenyAskVoteKick(byWhom, playerIdWho, KICK_DENIED_OTHER_IN_PROGRESS);
+			}
+			else
+				InternalDenyAskVoteKick(byWhom, playerIdWho, KICK_DENIED_INVALID_PLAYER_ID);
+		}
+		else
+			InternalDenyAskVoteKick(byWhom, playerIdWho, KICK_DENIED_TOO_FEW_PLAYERS);
+	}
+	else
+		InternalDenyAskVoteKick(byWhom, playerIdWho, KICK_DENIED_INVALID_STATE);
+}
+
+void
+ServerGameThread::InternalDenyAskVoteKick(SessionWrapper byWhom, unsigned playerIdWho, DenyKickPlayerReason reason)
+{
+	boost::shared_ptr<NetPacket> denyPetition(new NetPacketAskKickPlayerDenied);
+	NetPacketAskKickPlayerDenied::Data denyPetitionData;
+	denyPetitionData.playerId = playerIdWho;
+	denyPetitionData.denyReason = reason;
+	static_cast<NetPacketAskKickPlayerDenied *>(denyPetition.get())->SetData(denyPetitionData);
+	GetSender().Send(byWhom.sessionData, denyPetition);
+}
+
+void
+ServerGameThread::InternalVoteKick(SessionWrapper byWhom, unsigned petitionId, KickVote vote)
+{
+	if (IsRunning() && byWhom.playerData)
+	{
+		boost::mutex::scoped_lock lock(m_voteKickDataMutex);
+		// Check whether this is the valid petition id.
+		if (m_voteKickData->petitionId == petitionId)
+		{
+			// Check whether the player already voted.
+			unsigned playerId = byWhom.playerData->GetUniqueId();
+			if (find(m_voteKickData->votedPlayerIds.begin(), m_voteKickData->votedPlayerIds.end(), playerId) == m_voteKickData->votedPlayerIds.end())
+			{
+				m_voteKickData->votedPlayerIds.push_back(playerId);
+				if (vote == KICK_VOTE_IN_FAVOUR)
+					m_voteKickData->numVotesInFavourOfKicking++;
+				else
+					m_voteKickData->numVotesAgainstKicking++;
+				// Send update notification.
+				boost::shared_ptr<NetPacket> updatePetition(new NetPacketKickPlayerPetitionUpdate);
+				NetPacketKickPlayerPetitionUpdate::Data updatePetitionData;
+				updatePetitionData.petitionId = m_voteKickData->petitionId;
+				updatePetitionData.numVotesAgainstKicking = m_voteKickData->numVotesAgainstKicking;
+				updatePetitionData.numVotesInFavourOfKicking = m_voteKickData->numVotesInFavourOfKicking;
+				updatePetitionData.numVotesNeededToKick = m_voteKickData->numVotesToKick;
+
+				static_cast<NetPacketKickPlayerPetitionUpdate *>(updatePetition.get())->SetData(updatePetitionData);
+				SendToAllPlayers(updatePetition, SessionData::Game);
+			}
+			else
+				InternalDenyVoteKick(byWhom, petitionId, VOTE_DENIED_ALREADY_VOTED);
+		}
+		else
+			InternalDenyVoteKick(byWhom, petitionId, VOTE_DENIED_INVALID_PETITION);
+	}
+	else
+		InternalDenyVoteKick(byWhom, petitionId, VOTE_DENIED_IMPOSSIBLE);
+}
+
+void
+ServerGameThread::InternalDenyVoteKick(SessionWrapper byWhom, unsigned petitionId, DenyVoteReason reason)
+{
+	boost::shared_ptr<NetPacket> denyVote(new NetPacketVoteKickPlayerDenied);
+	NetPacketVoteKickPlayerDenied::Data denyVoteData;
+	denyVoteData.petitionId = petitionId;
+	denyVoteData.denyReason = reason;
+	static_cast<NetPacketVoteKickPlayerDenied *>(denyVote.get())->SetData(denyVoteData);
+	GetSender().Send(byWhom.sessionData, denyVote);
 }
 
 PlayerDataList
@@ -307,6 +519,45 @@ ServerGameThread::AddComputerPlayer(boost::shared_ptr<PlayerData> player)
 	GetLobbyThread().AddComputerPlayer(player);
 }
 
+boost::shared_ptr<PlayerData>
+ServerGameThread::RemoveComputerPlayer(unsigned playerId)
+{
+	boost::shared_ptr<PlayerData> tmpPlayer;
+	{
+		boost::mutex::scoped_lock lock(m_computerPlayerListMutex);
+		PlayerDataList::iterator i = m_computerPlayerList.begin();
+		PlayerDataList::iterator end = m_computerPlayerList.end();
+		while (i != end)
+		{
+			if ((*i)->GetUniqueId() == playerId)
+			{
+				tmpPlayer = *i;
+				m_computerPlayerList.erase(i);
+				break;
+			}
+			++i;
+		}
+	}
+	GetLobbyThread().RemoveComputerPlayer(tmpPlayer);
+	return tmpPlayer;
+}
+
+bool
+ServerGameThread::IsComputerPlayerActive(unsigned playerId) const
+{
+	bool retVal = false;
+	boost::mutex::scoped_lock lock(m_computerPlayerListMutex);
+	PlayerDataList::const_iterator i = m_computerPlayerList.begin();
+	PlayerDataList::const_iterator end = m_computerPlayerList.end();
+	while (i != end)
+	{
+		if ((*i)->GetUniqueId() == playerId)
+			retVal = true;
+		++i;
+	}
+	return retVal;
+}
+
 void
 ServerGameThread::ResetComputerPlayerList()
 {
@@ -318,7 +569,7 @@ ServerGameThread::ResetComputerPlayerList()
 	while (i != end)
 	{
 		GetLobbyThread().RemoveComputerPlayer(*i);
-		RemovePlayerData(*i);
+		RemovePlayerData(*i, NTF_NET_REMOVED_ON_REQUEST);
 		++i;
 	}
 
@@ -326,7 +577,7 @@ ServerGameThread::ResetComputerPlayerList()
 }
 
 void
-ServerGameThread::GracefulRemoveSession(SessionWrapper session)
+ServerGameThread::GracefulRemoveSession(SessionWrapper session, int reason)
 {
 	if (!session.sessionData.get())
 		throw ServerException(__FILE__, __LINE__, ERR_NET_INVALID_SESSION, 0);
@@ -335,12 +586,12 @@ ServerGameThread::GracefulRemoveSession(SessionWrapper session)
 	boost::shared_ptr<PlayerData> tmpPlayerData = session.playerData;
 	if (tmpPlayerData.get() && !tmpPlayerData->GetName().empty())
 	{
-		RemovePlayerData(tmpPlayerData);
+		RemovePlayerData(tmpPlayerData, reason);
 	}
 }
 
 void
-ServerGameThread::RemovePlayerData(boost::shared_ptr<PlayerData> player)
+ServerGameThread::RemovePlayerData(boost::shared_ptr<PlayerData> player, int reason)
 {
 	if (player->GetRights() == PLAYER_RIGHTS_ADMIN)
 	{
@@ -370,6 +621,7 @@ ServerGameThread::RemovePlayerData(boost::shared_ptr<PlayerData> player)
 	boost::shared_ptr<NetPacket> thisPlayerLeft(new NetPacketPlayerLeft);
 	NetPacketPlayerLeft::Data thisPlayerLeftData;
 	thisPlayerLeftData.playerId = player->GetUniqueId();
+	thisPlayerLeftData.removeReason = reason;
 	static_cast<NetPacketPlayerLeft *>(thisPlayerLeft.get())->SetData(thisPlayerLeftData);
 	GetSessionManager().SendToAllSessions(GetSender(), thisPlayerLeft, SessionData::Game);
 
@@ -380,7 +632,7 @@ void
 ServerGameThread::ErrorRemoveSession(SessionWrapper session)
 {
 	GetLobbyThread().RemoveSessionFromGame(session);
-	GracefulRemoveSession(session);
+	GracefulRemoveSession(session, NTF_NET_INTERNAL);
 }
 
 void
@@ -395,7 +647,9 @@ ServerGameThread::SessionError(SessionWrapper session, int errorCode)
 void
 ServerGameThread::MoveSessionToLobby(SessionWrapper session, int reason)
 {
-	GracefulRemoveSession(session);
+	GracefulRemoveSession(session, reason);
+	// Reset ready flag - just in case it is set, player may leave at any time.
+	session.sessionData->ResetReadyFlag();
 	GetLobbyThread().ReAddSession(session, reason);
 }
 
@@ -410,7 +664,8 @@ ServerGameThread::RemoveDisconnectedPlayers()
 		while (i != end)
 		{
 			boost::shared_ptr<PlayerInterface> tmpPlayer = *i;
-			if (!GetSessionManager().IsPlayerConnected(tmpPlayer->getMyUniqueID()) && tmpPlayer->getMyType() == PLAYER_TYPE_HUMAN)
+			if ((tmpPlayer->getMyType() == PLAYER_TYPE_HUMAN && !GetSessionManager().IsPlayerConnected(tmpPlayer->getMyUniqueID()))
+				|| (tmpPlayer->getMyType() == PLAYER_TYPE_COMPUTER && !IsComputerPlayerActive(tmpPlayer->getMyUniqueID())))
 			{
 				// Setting player cash to 0 will deactivate the player.
 				tmpPlayer->setMyCash(0);
@@ -444,6 +699,16 @@ ServerGameThread::AssignPlayerNumbers()
 	}
 }
 
+bool
+ServerGameThread::IsValidPlayer(unsigned playerId) const
+{
+	bool retVal = false;
+	const PlayerIdList list(GetPlayerIdList());
+	if (find(list.begin(), list.end(), playerId) != list.end())
+		retVal = true;
+	return retVal;
+}
+
 SessionManager &
 ServerGameThread::GetSessionManager()
 {
